// DAZ Studio Icon Handler.cpp : Implementation of DLL Exports.

#include "stdafx.h"
#include "resource.h"
#include "DAZStudioIconHandler_i.h"
#include "dllmain.h"

#include <shlobj.h>
#include <strsafe.h>

extern const TCHAR backupIconKeyName[] = _T("\\PreviousDefaultIcon");
static const TCHAR defaultIconKeyName[] = _T("\\DefaultIcon");
static const TCHAR defaultIconValue[] = _T("%1");

// Register Icon Handler
STDAPI RegisterIconHandler (LPTSTR extension, bool unregister = false)
{
    HRESULT hr = E_FAIL;
	HKEY key;
	LONG result;
	TCHAR keyBuffer[256 + 1];	// Max size of a key plus null
	TCHAR keyBufferDI[256 + sizeof (defaultIconKeyName) + 1];	// Max size of a key plus DefaultIcon key plus null
	TCHAR keyBufferBI[256 + sizeof (backupIconKeyName) + 1];	// Max size of a key plus BackupIcon key plus null
	TCHAR dataBuffer[MAX_PATH + 10];	// Max path size + space for icon position
	DWORD keyBufferSize = ARRAYSIZE(keyBuffer);
	DWORD dataBufferSize = ARRAYSIZE(dataBuffer);

	// Get location of default icon key for extension
	result = RegOpenKeyEx (HKEY_CLASSES_ROOT, extension, 0, KEY_READ, &key);
	if (result != ERROR_SUCCESS)
		goto failure;

	result = SHRegGetValue (key, NULL, NULL, SRRF_RT_REG_SZ, NULL, (LPBYTE)keyBuffer, &keyBufferSize);
	RegCloseKey (key);
	if (result != ERROR_SUCCESS)
		goto failure;

	result = StringCchCopy (keyBufferDI, ARRAYSIZE(keyBufferDI), keyBuffer);
	if (FAILED(result))
		goto failure;

	result = StringCchCat (keyBufferDI, ARRAYSIZE(keyBufferDI), defaultIconKeyName);
	if (FAILED(result))
		goto failure;

	// Get location of backup icon key
	result = StringCchCopy (keyBufferBI, ARRAYSIZE(keyBufferBI), keyBuffer);
	if (FAILED(result))
		goto failure;

	result = StringCchCat (keyBufferBI, ARRAYSIZE(keyBufferBI), backupIconKeyName);
	if (FAILED(result))
		goto failure;

	// Get value of current DefaultIcon
	result = SHRegGetValue (HKEY_CLASSES_ROOT, keyBufferDI, NULL, SRRF_RT_REG_SZ, NULL, (LPBYTE)dataBuffer, &dataBufferSize);
	if (result != ERROR_SUCCESS)
		dataBuffer[0] = _T('\0');

	// Backup the key and configure Icon Handler
	if (!unregister)
	{
		if (_tcscmp (dataBuffer, _T("%1")) != 0)
		{
			// Backup default icon key
			result = RegCreateKeyEx (HKEY_CLASSES_ROOT, keyBufferBI, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &key, NULL);
			if (result != ERROR_SUCCESS)
				goto failure;

			result = SHCopyKey (HKEY_CLASSES_ROOT, keyBufferDI, key, 0);
			RegCloseKey (key);
			if (result != ERROR_SUCCESS)
				goto failure;

			// Configure the Icon Handler
			result = SHSetValue (HKEY_CLASSES_ROOT, keyBufferDI, NULL, REG_SZ, defaultIconValue, sizeof (defaultIconValue));
			if (result != ERROR_SUCCESS)
				goto failure;
		}

		hr = S_OK;
	}
	// Restore the key and remove the backup
	else
	{
		if (_tcscmp (dataBuffer, _T("%1")) == 0)
		{
			result = RegOpenKeyEx (HKEY_CLASSES_ROOT, keyBufferDI, 0, KEY_SET_VALUE, &key);
			if (result != ERROR_SUCCESS)
				goto failure;

			// Restore the default icon key
			result = SHCopyKey (HKEY_CLASSES_ROOT, keyBufferBI, key, 0);
			RegCloseKey (key);
			if (result != ERROR_SUCCESS)
				goto failure;
		}

		// Remove the backup key
		result = RegDeleteKey (HKEY_CLASSES_ROOT, keyBufferBI);
		if (result != ERROR_SUCCESS)
			{ } // Don't care

		hr = S_OK;
	}

failure:
	return hr;
}

// Used to determine whether the DLL can be unloaded by OLE
STDAPI DllCanUnloadNow(void)
{
    return _AtlModule.DllCanUnloadNow();
}

// Returns a class factory to create an object of the requested type
STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID* ppv)
{
    return _AtlModule.DllGetClassObject(rclsid, riid, ppv);
}

// DllRegisterServer - Adds entries to the system registry
STDAPI DllRegisterServer(void)
{
    // registers object, typelib and all interfaces in typelib
    HRESULT hr = _AtlModule.DllRegisterServer();

	if (SUCCEEDED(hr))
	{
		// Register Icon Handler
		RegisterIconHandler(_T(".duf"));
		RegisterIconHandler(_T(".dsf"));
		RegisterIconHandler(_T(".daz"));
		RegisterIconHandler(_T(".dsa"));
		RegisterIconHandler(_T(".dsb"));
		RegisterIconHandler(_T(".dse"));
		RegisterIconHandler(_T(".ds"));

		// Notify shell that a file type association has changed
		SHChangeNotify (SHCNE_ASSOCCHANGED, SHCNF_IDLIST, NULL, NULL);
	}

	return hr;
}

// DllUnregisterServer - Removes entries from the system registry
STDAPI DllUnregisterServer(void)
{
	// Unregister Icon Handler
	RegisterIconHandler(_T(".duf"), true);
	RegisterIconHandler(_T(".dsf"), true);
	RegisterIconHandler(_T(".daz"), true);
	RegisterIconHandler(_T(".dsa"), true);
	RegisterIconHandler(_T(".dsb"), true);
	RegisterIconHandler(_T(".dse"), true);
	RegisterIconHandler(_T(".ds"), true);

	// Notify shell that a file type association has changed
	SHChangeNotify (SHCNE_ASSOCCHANGED, SHCNF_IDLIST, NULL, NULL);

	HRESULT hr = _AtlModule.DllUnregisterServer();

	return hr;
}

// DllInstall - Adds/Removes entries to the system registry per user
//              per machine.	
STDAPI DllInstall(BOOL bInstall, LPCWSTR pszCmdLine)
{
    HRESULT hr = E_FAIL;
    static const wchar_t szUserSwitch[] = _T("user");

    if (pszCmdLine != NULL)
    {
    	if (_wcsnicmp(pszCmdLine, szUserSwitch, _countof(szUserSwitch)) == 0)
    	{
    		AtlSetPerUserRegistration(true);
    	}
    }

    if (bInstall)
    {	
    	hr = DllRegisterServer();
    	if (FAILED(hr))
    	{	
    		DllUnregisterServer();
    	}
    }
    else
    {
    	hr = DllUnregisterServer();
    }

    return hr;
}