// DSIH.cpp : Implementation of CDSIH

#include "stdafx.h"
#include "DSIH.h"

#include <strsafe.h>

#pragma comment (lib, "gdiplus.lib")

extern const TCHAR backupIconKeyName[];

// CDSIH
CDSIH::CDSIH()
{
	// Start GDI+
	Gdiplus::GdiplusStartupInput gdiplusStartupInput;
	Gdiplus::GdiplusStartup (&m_gdiplusToken, &gdiplusStartupInput, NULL);
}

CDSIH::~CDSIH()
{
	// Stop GDI+
	Gdiplus::GdiplusShutdown (m_gdiplusToken);
}

STDMETHODIMP CDSIH::GetClassID (CLSID *pClassID)
{
	*pClassID = CLSID_DSIH;
	return S_OK;
}

STDMETHODIMP CDSIH::Load (LPCOLESTR pszFile, DWORD dwMode)
{
	UNREFERENCED_PARAMETER (dwMode);

	LPCTSTR extension;

	// Get the icon to show if there is a problem showing the associated PNG
	extension = _tcsrchr (pszFile, _T('.'));
	getDefaultIcon (extension);

	// Save filename
	return StringCchCopy (m_szFileName, ARRAYSIZE(m_szFileName), pszFile);
}

STDMETHODIMP CDSIH::GetIconLocation (UINT uFlags, LPTSTR szIconFileName, UINT cchMax, int *piIndex, UINT *pwFlags)
{
	// Read the following thread before modifying this function
	// http://groups.google.com/group/microsoft.public.platformsdk.shell/browse_thread/thread/8b2714e6f59e5066/523b5b5112e546d4

	UNREFERENCED_PARAMETER (uFlags);

	HRESULT result;
	TCHAR szPngFileName[MAX_PATH];

	// Generate png filename
	result = StringCchCopy (szPngFileName, ARRAYSIZE(szPngFileName), m_szFileName);
	if (FAILED(result))
		goto failure;

	result = StringCchCat (szPngFileName, ARRAYSIZE(szPngFileName), _T(".png"));
	if (FAILED(result))
		goto failure;

	// Test for existance of png file
	HANDLE handle = CreateFile (szPngFileName, GENERIC_READ, FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (handle != INVALID_HANDLE_VALUE)
	{
		CloseHandle (handle);

		// Set piIndex and szIconFile so they can be used for caching
		*piIndex = 0;
		result = StringCchCopy (szIconFileName, cchMax, szPngFileName);
		*pwFlags = GIL_NOTFILENAME;

//#ifdef _DEBUG
//		*pwFlags |= GIL_DONTCACHE;
//#endif

		// Need this if extracting the icons is slow - Not sure yet
		//	if (uFlags == (uFlags & GIL_ASYNC))
		//			return E_PENDING;
	}

	return S_OK;

failure:
	//  Set szIconFile and piIndex to the default DAZ file icon
	*piIndex = m_iDefaultIconPosition;
	result = StringCchCopy (szIconFileName, cchMax, m_szDefaultIconLocation);
	if (FAILED(result))
		{ } // Do nothing

	return S_FALSE;
}

STDMETHODIMP CDSIH::Extract (LPCTSTR pszFile, UINT nIconIndex, HICON *phiconLarge, HICON *phiconSmall, UINT nIconSize)
{
	UNREFERENCED_PARAMETER (pszFile);
	UNREFERENCED_PARAMETER (nIconIndex);

	// Load file to make icon from
	Gdiplus::Image image (pszFile);
	if (image.GetLastStatus() == Gdiplus::Ok)
	{
		HRESULT result;

		// Get large icon
		result = ImageToIcon (&image, phiconLarge, min (LOWORD(nIconSize), image.GetWidth()));
		if (FAILED(result))
			phiconLarge = NULL;

		// Get small icon
		result = ImageToIcon (&image, phiconSmall, min (HIWORD(nIconSize), image.GetWidth()));
		if (FAILED(result))
			phiconSmall = NULL;

		// If either or both icons were retrieved return success
		if (phiconLarge || phiconSmall)
			return S_OK;
	}

	// Try to extract the icon
	int numIcons = ExtractIconEx (m_szDefaultIconLocation, m_iDefaultIconPosition, phiconLarge, phiconSmall, 1);
	if (numIcons)
		return S_OK;

	return S_FALSE;
}

STDMETHODIMP CDSIH::ImageToIcon (Gdiplus::Image* image, HICON* icon, int size)
{
	using namespace Gdiplus;

	Status status;

	// Create a new bitmap and setup graphic access
	Bitmap bitmap (size, size);
	Graphics bitmapGraphic (&bitmap);

	// Configure GDI+ to resample quickly but with high quality
	// Nothing special needs to be done if any of these fail
	bitmapGraphic.SetCompositingMode (CompositingModeSourceCopy);
    bitmapGraphic.SetCompositingQuality (CompositingQualityHighSpeed);
    bitmapGraphic.SetPixelOffsetMode (PixelOffsetModeHighSpeed);
    bitmapGraphic.SetSmoothingMode (SmoothingModeHighSpeed);
    bitmapGraphic.SetInterpolationMode (InterpolationModeHighQuality);

	// Copy and scale the supplied image into the new bitmap
	status = bitmapGraphic.DrawImage (image, 0, 0, size, size);
	if (status != Ok)
		goto failure;

	// Get the new icon
	status = bitmap.GetHICON (icon);
	if (status != Ok)
		goto failure;

	return S_OK;

failure:
	return E_FAIL;
}

void CDSIH::getDefaultIcon (LPCTSTR extension)
{
	HKEY key;
	LONG result;
	TCHAR keyBuffer[513];	// Max size of two keys plus two backslashes plus null
	TCHAR dataBuffer[MAX_PATH + 10];	// Max path size + space for icon position
	DWORD keyBufferSize = ARRAYSIZE(keyBuffer);
	DWORD dataBufferSize = ARRAYSIZE(dataBuffer);
	LPTSTR seperatorPosition;

	// Get location of the backup default icon key for extension
	result = RegOpenKeyEx (HKEY_CLASSES_ROOT, extension, 0, KEY_READ, &key);
	if (result != ERROR_SUCCESS)
		goto failure;

	result = SHRegGetValue (key, NULL, NULL, SRRF_RT_REG_SZ, NULL, (LPBYTE)keyBuffer, &keyBufferSize);
	RegCloseKey (key);
	if (result != ERROR_SUCCESS)
		goto failure;

	result = StringCchCat (keyBuffer, ARRAYSIZE(keyBuffer), backupIconKeyName);
	if (FAILED(result))
		goto failure;

	result = RegOpenKeyEx (HKEY_CLASSES_ROOT, keyBuffer, 0, KEY_READ, &key);
	if (result != ERROR_SUCCESS)
		goto failure;

	// Get the backup default icon file and icon position
	result = SHRegGetValue (key, NULL, NULL, SRRF_RT_REG_SZ, NULL, (LPBYTE)dataBuffer, &dataBufferSize);
	RegCloseKey (key);
	if (result != ERROR_SUCCESS)
		goto failure;

	// Separate the filename and position
	seperatorPosition = _tcsrchr (dataBuffer, _T(','));
	if (seperatorPosition)
	{
		*seperatorPosition++ = _T('\0');
		m_iDefaultIconPosition = _ttoi (seperatorPosition);
	}
	else
		m_iDefaultIconPosition = 0;

	result = StringCchCopy (m_szDefaultIconLocation, ARRAYSIZE(m_szDefaultIconLocation), dataBuffer);
	if (FAILED(result))
		goto failure;

	return;	// Successful

failure:
	m_iDefaultIconPosition = 0;
	m_szDefaultIconLocation[0] = '\0';
}